@@ -2,7 +2,7 @@
 # 
 # Make simple Subversion revision merges and branch switching much easier.
 #
-# Copyright (c) 2006--2009 Philipp von Weitershausen, Marius Gedminas
+# Copyright (c) 2006--2010 Philipp von Weitershausen, Marius Gedminas
 #
 # This program is distributed under the terms of the GNU General Public Licence
 # See the file COPYING for details.
@@ -21,7 +21,7 @@ import subprocess
 from xml.dom import minidom
 
 
-VERSION = '1.11.0'
+VERSION = '1.12.1'
 
 
 #
@@ -210,6 +210,20 @@ def determinebranch(branch, path, svninfo=svninfo):
       >>> determinebranch('tag/3.4', '.', svninfo=dummyinfo)
       'http://dev.worldcookery.com/svn/bla/tag/3.4/blergh'
 
+      Switching from a tag to a branch:
+
+      >>> def dummyinfo(path):
+      ...     return '''\
+      ... Path: .
+      ... URL: http://dev.worldcookery.com/svn/bla/tags/foobar
+      ... Repository UUID: ab69c8a2-bfcb-0310-9bff-acb20127a769
+      ... Revision: 1654
+      ... Node Kind: directory
+      ... '''
+
+      >>> determinebranch('mybranch', '.', svninfo=dummyinfo)
+      'http://dev.worldcookery.com/svn/bla/branches/mybranch'
+
     """
     lines = svninfo(path).splitlines()
     if lines[1].startswith('URL: '):
@@ -230,7 +244,10 @@ def determinebranch(branch, path, svninfo=svninfo):
                 new_chunks.append(branch)
             else:
                 if not is_tag:
-                    new_chunks.append(ch)
+                    if ch in ('tag', 'tags'):
+                        new_chunks.append('branches')
+                    else:
+                        new_chunks.append(ch)
                 new_chunks.append(branch)
         elif ch == 'trunk' and branch != 'trunk':
             if not is_tag:
@@ -482,8 +499,8 @@ def command(cmd, help_msg, alias=None):
 def ezmerge(argv, progname=None):
     progname = progname or os.path.basename(argv[0])
     parser = optparse.OptionParser(
-                "usage: %prog [options] [rev] source-branch [wc-path]"
-                "       %prog -l\n",
+                "usage: %prog [options] [rev] source-branch [wc-path]\n"
+                "       %prog -l",
                 prog=progname,
                 description="merge changes from Subversion branches")
     parser.add_option('-l', '--list',
@@ -495,6 +512,9 @@ def ezmerge(argv, progname=None):
     parser.add_option('-d', '--diff',
                       help='show a diff of changes on the branch',
                       action='store_true', dest='diff', default=False)
+    parser.add_option('-t', '--tag',
+                      help='use a tag instead of a branch',
+                      action='store_true', dest='tag', default=False)
     parser.add_option('-n', '--dry-run',
                       help='do not touch any files on disk or in subversion',
                       action='store_true', dest='dry_run', default=False)
@@ -510,7 +530,10 @@ def ezmerge(argv, progname=None):
 
     if opts.list_branches:
         # TODO: allow a different wc-path
-        print '\n'.join(listbranches('.'))
+        if opts.tag:
+            print '\n'.join(listtags('.'))
+        else:
+            print '\n'.join(listbranches('.'))
         return
 
     if len(args) == 1:
@@ -523,9 +546,15 @@ def ezmerge(argv, progname=None):
     if len(args) > 2:
         path = args[2]
 
-    branch = determinebranch(branchname, path)
+    if opts.tag:
+        branch = determinetag(branchname, path)
+    else:
+        branch = determinebranch(branchname, path)
     if branchname != 'trunk' and not branchname.endswith('branch'):
-        branchname += ' branch'
+        if opts.tag:
+            branchname += ' tag'
+        else:
+            branchname += ' branch'
     if rev == 'ALL':
         beginrev, endrev = branchpoints(branch)
         if branchname == 'trunk':
@@ -543,9 +572,11 @@ def ezmerge(argv, progname=None):
     if opts.diff:
         merge_cmd = "svn diff -r %s:%s %s" % (beginrev, endrev, branch)
     else:
-        reintegrate = opts.reintegrate and " --reintegrate" or ""
-        merge_cmd = "svn merge%s -r %s:%s %s %s" % (
-            reintegrate, beginrev, endrev, branch, path)
+        if opts.reintegrate:
+            revisions = "--reintegrate"
+        else:
+            revisions = "-r %s:%s" % (beginrev, endrev)
+        merge_cmd = "svn merge %s %s %s" % (revisions, branch, path)
     if not opts.diff:
         print msg, "with"
         print
@@ -665,6 +696,12 @@ def ezswitch(argv, progname=None):
     branch = determinebranch(branch, path)
     if opts.create_branch:
         cur_branch = currentbranch(path)
+        if cur_branch == branch:
+            # XXX determinebranch() should raise this kind of error
+            # instead of me doing it here
+            sys.exit("can't figure out the branch structure of %s\n"
+                     "expected a path element named trunk/branches/tags/branch/tag"
+                     % branch)
         cmd = "svn cp %s %s" % (cur_branch, branch)
         if opts.message:
             cmd += " -m '%s'" % opts.message
@@ -772,7 +809,7 @@ def rmbranch(argv, progname=None):
     progname = progname or os.path.basename(argv[0])
     parser = optparse.OptionParser(
                 "usage: %prog [-n] [-m MSG] branch\n"
-                "       %prog -l\n",
+                "       %prog -l",
                 prog=progname,
                 description="Remove a named Subversion branch.")
     parser.add_option('-l', '--list',
@@ -815,7 +852,7 @@ def mvbranch(argv, progname=None):
     progname = progname or os.path.basename(argv[0])
     parser = optparse.OptionParser(
                 "usage: %prog [-n] [-m MSG] oldbranch newbranch\n"
-                "       %prog -l\n",
+                "       %prog -l",
                 prog=progname,
                 description="Rename a Subversion branch.")
     parser.add_option('-l', '--list',
@@ -859,7 +896,7 @@ def branchdiff(argv, progname=None):
     progname = progname or os.path.basename(argv[0])
     parser = optparse.OptionParser(
                 "usage: %prog [options] [branch [wc-path]]"
-                "       %prog -l\n",
+                "       %prog -l",
                 prog=progname,
                 description="show combined diff of all changes made on a branch")
     parser.add_option('-l', '--list',
@@ -888,7 +925,7 @@ def branchpoint(argv, progname=None):
     progname = progname or os.path.basename(argv[0])
     parser = optparse.OptionParser(
                 "usage: %prog [options] [branch [wc-path]]"
-                "       %prog -l\n",
+                "       %prog -l",
                 prog=progname,
                 description="show the revision number when a branch was created")
     parser.add_option('-l', '--list',
